create or replace procedure who_sql_xref
(
 XPsenha in out varchar2
 )
as

PRAGMA AUTONOMOUS_TRANSACTION;


--*====================================================================
--* Procedure..:  who_SQL_xref (Adaptao do Modelo da who_java_xref)
--* Descricao..:  Atualiza a Tabela Z_SQL_SQL_XREF com as referncias cruzadas
--*               das Tabelas
--*               USER_TABLE, user_views, user_mviews, user_sequences,
--*               USER_SYNONYMS,
--*               USER_PROCEDURES,
--*               alm de COMANDOS SQL, incluindo classes JPA Provider,
--*               e rastreamento classes Action pelo cruzamento com a Tabela
--*               SEG_ROTINAS que abrigam as URLs das EAPs e a Z_JAVA_SQL_XREF.
--*
--*                um PRAGMA AUTONOMOUS_TRANSACTION onde seu commit no
--*               interfere na lgica de nenhuma outra aplicao.
--*               IMPORTANTE:
--*               ==========
--*
--*               Existe apenas um parmetro de senha de execuo (IN/OUT).
--*               No existe uma aplicao JAVA que recebe o(s) Diretrio(s)  partir
--*               do(s) qual(ais) sero extrados TODOS os arquivos FONTE SQL 
--*               como no modelo JAVA (.java; .xml) em que esta procedure se adaptou,
--*               mas esta procedure efetua a carga de TODOS FONTES SQL via USER_SOURCE
--*               fazendo INSERT na Tabela Z_SQL_SQL_XREF removendo duplicidade pela chave:
--*
--*               NMARQJAV - Nome do Arquivo SQL (contem o NOME OBJETO SOURCE SQL)
--*                          que pode tambm estar relacionado com ARQuivo fontes JAVa. 
--*
--*               A outra componente da chave, NRLIN, fica como segurana desta aplicao e
--*               como facilitador de ordenao do fonte SQL.
--*
--*               O RESET na Z_SQL_SQL_XREF (DELETE) SEMPRE  feito ao se executar esta procedure
--*               "refazendo" o XREF devido o acmulo de manutenes nas estruturas SQL e/ou
--*               nos fontes JAVA (espera-se execuo previa da procedure modelo WHO_JAVA_XREF).
--*
--*               Ao final com o comando SQL abaixo  extraida uma Planilha Excel com as referncias
--*               SQL presentes nos fontes SQL e JAVA permitindo um rastreamento simples por Auto-Filtro.
--*               Esta procedure faz uso da Z_JAVA_SQL_XREF reforando rastreamento entre as referncias
--*               cruzadas dos fontes JAVA e SQL estabelecendo semelhana pela ID da funcionalidade-EAP
--*               e Classe Action Java (procedure/function) e Classe JPA (Tables/Views/Seqs/Synonyms).
--*
--*               Select * from Z_SQL_SQL_XREF where
--*               RCTABVIESEQ is not null or
--*               RCPROFUN    is not null or
--*               RCSQLCMD    is not null or
--*               RCSYN       is not null or
--*               RCCLAJPA    is not null or
--*               RCCLAACT    is not null;
--*
--*               Z_SQL_SQL_XREF
--*
--*               NMARQJAV  VARCHAR2(2048)
--*               NRLIN  NUMBER(8)
--*               DCLIN  VARCHAR2(4000)
--*               RCTABVIESEQ  VARCHAR2(2048)
--*               RCPROFUN  VARCHAR2(2048)
--*               RCSQLCMD  VARCHAR2(2048)
--*               RCSYN  VARCHAR2(2048)
--*               RCCLAJPA  VARCHAR2(2048)
--*               RCCLAACT  VARCHAR2(2048)
--*
--* Revises...:  Criacao da Procedure
--* Data.......:  11/10/2013
--* Autor .....:  Renato Viana
--* Revises...:
--* Data.......:
--* Autor Rev..:
--*=====================================================================

  XPfetchs    number(8);
  XPTAGROWID  ROWID;
  XPNMTAB     VARCHAR2(20);
  XP_JA_OUT   VARCHAR2(2048);
  XPEXCLEN    VARCHAR2(20);
  XPEXCOK     VARCHAR2(20);


------------ "Record Fetch" Cursor
XNMOBJ        VARCHAR2(256);
Xnmarqjav     VARCHAR2(2048);
Xnrlin        NUMBER(8);
Xdclin        VARCHAR2(4000);
Xrctabvieseq  VARCHAR2(2048);
Xrcprofun     VARCHAR2(2048);
Xrcsqlcmd     VARCHAR2(2048);
Xrcsyn        VARCHAR2(2048);
Xrcclajpa     VARCHAR2(2048);
Xrcclaact     VARCHAR2(2048);
Xrowid        ROWID;

------------ Cursores


cursor sqlTABLES is
SELECT
B.TABLE_NAME,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
USER_TABLES B
WHERE A.DCLIN is not null
AND UPPER(A.DCLIN) LIKE '%'||B.TABLE_NAME||'%'

order by a.nmarqjav, a.nrlin;

cursor sqlVIEW is
SELECT
B.VIEW_NAME,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
user_views B
WHERE A.DCLIN is not null
AND UPPER(A.DCLIN) LIKE '%'||B.VIEW_NAME||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlMVIEW is
SELECT
B.MVIEW_NAME,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
user_mviews B
WHERE A.DCLIN is not null
AND UPPER(A.DCLIN) LIKE '%'||B.MVIEW_NAME||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlSEQUENCES is
SELECT
B.SEQUENCE_NAME,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
user_sequences B
WHERE A.DCLIN is not null
AND UPPER(A.DCLIN) LIKE '%'||B.SEQUENCE_NAME||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlSYNONYMS is
SELECT
B.SYNONYM_NAME,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
USER_SYNONYMS B
WHERE A.DCLIN is not null
AND UPPER(A.DCLIN) LIKE '%'||B.SYNONYM_NAME||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlPROCEDURES is
SELECT
B.object_name,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid


FROM Z_SQL_SQL_XREF A,
USER_PROCEDURES B
WHERE A.DCLIN is not null
AND B.PROCEDURE_NAME is null
AND   UPPER(A.DCLIN) LIKE '%'||B.object_name||'%'
order by a.nmarqjav, a.nrlin;


/* --- uso somente no modelo WHO_JAVA_XREF
cursor sqlCLAJPA is
SELECT
A.rctabvieseq,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A


WHERE A.DCLIN is not null
AND a.rcclajpa is null
AND UPPER(a.dclin) LIKE '%@%.CLASS%TABLE%'
order by a.nmarqjav, a.nrlin;

*/

cursor sqlCKCLAJPA is
SELECT
B.rcclajpa,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
B.rctabvieseq,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
(SELECT rcclajpa, rctabvieseq, rcsyn from Z_JAVA_SQL_XREF
WHERE rcclajpa is not null
AND rctabvieseq is not null) B

WHERE A.rcclajpa is null
AND A.rctabvieseq is not null
AND A.rctabvieseq LIKE '%'||B.rctabvieseq||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlCLAACT is
SELECT
B.rcclaact,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
B.rcprofun,
A.rcclajpa,
A.rcclaact,
A.rowid

FROM Z_SQL_SQL_XREF A,
(select distinct Z.DCROT AS RCCLAACT,rcprofun from Z_JAVA_SQL_XREF,
z_Classeaction Z
 
WHERE rcprofun is not null
and UPPER(NMARQJAV) like '%'||UPPER(Z.CLASSEACTION)||'%') B

WHERE A.rcclaact is null
AND A.rcprofun is not null
AND A.rcprofun LIKE '%'||B.rcprofun||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlJPACOMMANDS is
SELECT
B.SQL_COMMAND,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid


FROM Z_SQL_SQL_XREF A,
Z_JPACOMMANDS B
WHERE A.DCLIN is not null
AND A.rcclajpa is not null
AND   UPPER(A.DCLIN) LIKE '%'||B.JPACOMMANDS_NAME||'%'
order by a.nmarqjav, a.nrlin;

cursor sqlCOMMANDS is
SELECT
B.SQL_COMMAND,
A.nmarqjav,
A.nrlin,
A.dclin,
A.rctabvieseq,
A.rcprofun,
A.rcsqlcmd,
A.rcsyn,
A.rcclajpa,
A.rcclaact,
A.rowid


FROM Z_SQL_SQL_XREF A,
Z_SQLCOMMANDS B
WHERE A.DCLIN is not null
AND A.rcclajpa is null
AND   UPPER(A.DCLIN) LIKE '%'||B.SQL_COMMAND||'%'
order by a.nmarqjav, a.nrlin;

-------- Ja existindo informao agrega com vrgula, alm de impedir duplicidade
FUNCTION Z_JA_XNMOBJ(
  XZ_JA_OLD IN VARCHAR2,
  XZ_JA_NEW IN VARCHAR2 )
   RETURN VARCHAR2 AS
   XZ_JA_OUT VARCHAR2(2048);
     BEGIN

     if XZ_JA_OLD is null then
       RETURN XZ_JA_NEW;
     end if;

     if XZ_JA_NEW is null then
       RETURN XZ_JA_OLD;
     end if;


     XZ_JA_OUT := XZ_JA_OLD || ',' || XZ_JA_NEW;
     RETURN XZ_JA_OUT;

     EXCEPTION
     WHEN OTHERS THEN
          XPEXCLEN := 'OBJ LENGTH > '|| XPNMTAB;
          RETURN XZ_JA_OLD;

    END Z_JA_XNMOBJ;

-------- Informao ObjectSQL User_Tables realmente corresponde ao LIKE %
FUNCTION Z_OK_XNMOBJ (
  XZ_OK_DCLIN IN VARCHAR2,
  XZ_OK_XNMOBJ IN VARCHAR2 )
   RETURN BOOLEAN AS
   XZ_OK_EXISTE NUMBER := 0;
   XZ_OK_ONDE   NUMBER := 1;
   XZ_OK_VEZ    NUMBER := 1;
   XZ_OK_FIM    NUMBER := 1;
   XZ_OK_LEN    NUMBER := 0;

     BEGIN

     if XZ_OK_DCLIN is null
     or XZ_OK_XNMOBJ is null then
       RETURN FALSE;
     end if;

     if length(XZ_OK_XNMOBJ) > length(XZ_OK_DCLIN) then
       RETURN FALSE;
     end if;

     XZ_OK_FIM := length(XZ_OK_DCLIN);
     XZ_OK_LEN := length(XZ_OK_XNMOBJ);


     LOOP

       XZ_OK_ONDE := INSTR(UPPER(XZ_OK_DCLIN),XZ_OK_XNMOBJ,XZ_OK_ONDE,XZ_OK_VEZ);
       
       IF XZ_OK_ONDE > 0 THEN
         IF (XZ_OK_ONDE + XZ_OK_LEN - 1) = XZ_OK_FIM
          OR XZ_OK_XNMOBJ || CHR(10) = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || ' ' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || '"' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || '''' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || ',' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || ';' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || ')' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || '(' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1))
          OR XZ_OK_XNMOBJ || '.' = UPPER(SUBSTR(XZ_OK_DCLIN,XZ_OK_ONDE,XZ_OK_LEN + 1)) -- 19/9/2013
         THEN
           XZ_OK_EXISTE := 1;
           RETURN TRUE;
         ELSE
           XZ_OK_VEZ := 2;
         END IF;
       ELSE
         XZ_OK_EXISTE := 0;
         RETURN FALSE;
       END IF;

     END LOOP;

     RETURN FALSE;

     EXCEPTION
     WHEN OTHERS THEN
          XPEXCOK := 'EXCEP OK OBJ ' || XPNMTAB;
          RETURN FALSE;

    END Z_OK_XNMOBJ;


begin

If XPsenha <> 'pass69' then
   XPsenha := 'Err pass';
   return;
end if;

--- SEMPRE REMOVE Z_SQL_SQL_XREF

    DELETE Z_SQL_SQL_XREF;
    COMMIT;
    
    INSERT INTO Z_SQL_SQL_XREF
    
    ( NMARQJAV,
      NRLIN,
      DCLIN )
    
    
    
    (SELECT A.NAME || 
     decode(A.TYPE, 'PROCEDURE', '.prc', 'FUNCTION', '.fnc', 'PACKAGE', '.pkg',
               'PACKAGE BODY', '.pkb', 'TRIGGER', '.trg', 'TYPE', '.typ', 'TYPE BODY', '.tyb',
               'SOURCE JAVA', '.java', '.und'),
            NVL(A.LINE,ROWNUM),   
            A.TEXT
      FROM USER_SOURCE A
      WHERE A.NAME IS NOT NULL
      /*AND A.NAME = 'MEPBT050' OR A.NAME = 'MEPBT068' OR A.NAME = 'MEPB0006'*/
      );
               
     COMMIT;          

--- Remove e Recria Tabelas Bsicas de Classe Action e Commands SQL
--- ===============================================================
delete z_classeaction;
--- A tabela z_classeaction
--- Aqui Z_ROTINAS corresponde a uma tabela do Sistema em que se faz XREF contendo colunas:
--- DCURLROT = Descrio URL da Rotina que contem path da rotina JAVA que liga um Menu Web
---            responsvel pelo "disparo" das funcionalidades (Action) e que normalmente so
---            utilizados no nome das demais classes diretamente ligadas a funcionalida;
--- IDSIS    = Identificador do Sistema em que se faz XREF;
--- DCROT    = Descrio da Rotina Funcional (ID da EAP).
insert into z_classeaction
(CLASSEACTION, DCURLROT, DCROT)
(select
case
when INSTR(Z_GET_FILENAME(SUBSTR(dcurlrot,1,(INSTR(dcurlrot,'.do?evento',-1,1)-1))),'man') > 0
then

SUBSTR(Z_GET_FILENAME(SUBSTR(dcurlrot,1,(INSTR(dcurlrot,'.do?evento',-1,1)-1))),1,
(INSTR((Z_GET_FILENAME(SUBSTR(dcurlrot,1,(INSTR(dcurlrot,'.do?evento',-1,1)-1)))),'man',-1,1)-1))

else

Z_GET_FILENAME(SUBSTR(dcurlrot,1,(INSTR(dcurlrot,'.do?evento',-1,1)-1)))

end AS CLASSEACTION
,dcurlrot
,dcrot from SEG_ROTINAS where dcurlrot is not null
 and idsis = 36
 and dcurlrot like '%?evento%'
 );
commit;

--- Outras Tabelas Mencionadas nos Cursores:
--- =======================================
--- Estas tabelas no so removidas ou recriadas aqui neste cdigo.
--- Elas so mantidas externamente em funo das necessidades de rastreamento.
---
--- Z_JPACOMMANDS
---
--- JPACOMMANDS_NAME  SQLCOMMAND
--- ================  ==========
--- .UPDATE           UPDATE
--- .MERGE            UPDATE
--- .SAVE             UPDATE
--- .PERSIST          UPDATE
--- .DELETE           DELETE
---
--- Z_SQLCOMMANDS
---
--- SQL_COMMAND
--- ===========
--- CREATE
--- DELETE
--- DROP
--- GRANT
--- INSERT
--- JOIN
--- SELECT
--- UNION
--- UPDATE
--- ALTER


----- Begin Cursor TABLES
XPNMTAB := 'sqlTABLES';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlTABLES;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlTABLES INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlTABLES%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rctabvieseq
INTO  Xrctabvieseq
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrctabvieseq,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrctabvieseq,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rctabvieseq = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlTABLES INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlTABLES;
----- END Cursor TABLES


----- Begin Cursor VIEW
XPNMTAB := 'sqlVIEW';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlVIEW;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlVIEW INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlVIEW%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rctabvieseq
INTO  Xrctabvieseq
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrctabvieseq,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrctabvieseq,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rctabvieseq = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlVIEW INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlVIEW;
----- END Cursor VIEW


----- Begin Cursor MVIEW (O uso deste cursor pode ser dispensado qdo TABLES o contiver)
XPNMTAB := 'sqlMVIEW';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlMVIEW;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlMVIEW INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlMVIEW%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rctabvieseq
INTO  Xrctabvieseq
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrctabvieseq,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrctabvieseq,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rctabvieseq = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlMVIEW INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlMVIEW;
----- END Cursor MVIEW


----- Begin Cursor sqlSEQUENCES
XPNMTAB := 'sqlSEQUENCES';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlSEQUENCES;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlSEQUENCES INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlSEQUENCES%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rctabvieseq
INTO  Xrctabvieseq
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrctabvieseq,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrctabvieseq,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rctabvieseq = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlSEQUENCES INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlSEQUENCES;
----- END Cursor sqlSEQUENCES


----- Begin Cursor sqlSYNONYMS
XPNMTAB := 'sqlSYNONYMS';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlSYNONYMS;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlSYNONYMS INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlSYNONYMS%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcsyn
INTO  Xrcsyn
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcsyn,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcsyn,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcsyn = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlSYNONYMS INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlSYNONYMS;
----- END Cursor sqlSYNONYMS


----- Begin Cursor sqlPROCEDURES
XPNMTAB := 'sqlPROCEDURES';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlPROCEDURES;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlPROCEDURES INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlPROCEDURES%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcprofun
INTO  Xrcprofun
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcprofun,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcprofun,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcprofun = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlPROCEDURES INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlPROCEDURES;
----- END Cursor sqlPROCEDURES


/* -- Uso apenas no modelo WHO_JAVA_XREF
----- Begin Cursor sqlCLAJPA
XPNMTAB := 'sqlCLAJPA';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlCLAJPA;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlCLAJPA INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlCLAJPA%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcclajpa
INTO  Xrcclajpa
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,Xrctabvieseq)
OR Z_OK_XNMOBJ(Xdclin,Xrcsyn) THEN

--- Cria o OBJ JPA utilizando a nomenclatura do arquivo
XNMOBJ := UPPER(Z_get_left_tag(z_get_filename(Xnmarqjav),'.'));

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcclajpa,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcclajpa,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcclajpa = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlCLAJPA INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlCLAJPA;
----- END Cursor sqlCLAJPA
*/

----- Begin Cursor sqlCKCLAJPA (Onde a CLAJPA montada anteriormente possui referencia)
XPNMTAB := 'sqlCKCLAJPA';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlCKCLAJPA;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlCKCLAJPA INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlCKCLAJPA%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcclajpa
INTO  Xrcclajpa
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- NO Verifica se o OBJ  Realmente Vlido
--- === pois, aqui rastrear proximidade (LIKE) com a classe JPA realizada pelas
--- === rctabvieseq apurada no JAVA com as rctabvieseq apuradas no SQL
--- === o OBJ POSICIONADO  o ID DA EAP correlacionada a classe ACTION

IF Z_OK_XNMOBJ(XNMOBJ,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcclajpa,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcclajpa,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcclajpa = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlCKCLAJPA INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlCKCLAJPA;
----- END Cursor sqlCKCLAJPA

<<CLASSE>>

----- Begin Cursor sqlCLAACT
XPNMTAB := 'sqlCLAACT';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlCLAACT;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlCLAACT INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlCLAACT%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

XNMOBJ := UPPER(XNMOBJ);
--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcclaact
INTO  Xrcclaact
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- NO Verifica se o OBJ  Realmente Vlido
--- === pois, aqui rastrear proximidade (LIKE) com a classe ACTION realizada pelas
--- === rcprofun apurada no JAVA com as rcprofun apuradas no SQL
--- === o OBJ POSICIONADO  o ID DA EAP correlacionada a classe ACTION
IF Z_OK_XNMOBJ(XNMOBJ,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcclaact,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcclaact,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcclaact = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlCLAACT INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlCLAACT;
----- END Cursor sqlCLAACT


----- Begin Cursor sqlJPACOMMANDS
XPNMTAB := 'sqlJPACOMMANDS';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlJPACOMMANDS;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlJPACOMMANDS INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlJPACOMMANDS%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

XNMOBJ := UPPER(XNMOBJ);
--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcsqlcmd
INTO  Xrcsqlcmd
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- NO Verifica se o OBJ  Realmente Vlido
--- === pois, aqui a finalidade  rastrear proximidade (LIKE) com a commands JPA
--- === afinal, pode existir no SQL fontes na linguagem JAVA
--- === alm do que, o OBJ POSICIONADO  um command SQL
IF Z_OK_XNMOBJ(XNMOBJ,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcsqlcmd,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcsqlcmd,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcsqlcmd = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlJPACOMMANDS INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlJPACOMMANDS;
----- END Cursor sqlJPACOMMANDS



----- Begin Cursor sqlCOMMANDS
XPNMTAB := 'sqlCOMMANDS';
XPTAGROWID := null;
XPfetchs := 0;
XPsenha := 'Open ' || XPNMTAB;
OPEN sqlCOMMANDS;
XPsenha := 'First Fetch ' || XPNMTAB;
FETCH sqlCOMMANDS INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

XPsenha := 'Question END ' || XPNMTAB;

WHILE sqlCOMMANDS%FOUND LOOP

XPfetchs := XPfetchs + 1;
XPsenha := 'Fetch ' || XPNMTAB || ' ' || XPfetchs;

--- Verifica necessidade de Recuperao Fecth devido a um possvel UPDATE
if XPTAGROWID = Xrowid then
XPsenha := 'Select ' || XPNMTAB || ' ' || XPfetchs;
SELECT rcsqlcmd
INTO  Xrcsqlcmd
FROM Z_SQL_SQL_XREF
WHERE ROWID = Xrowid;
end if;

--- Verifica se o OBJ  Realmente Vlido
IF Z_OK_XNMOBJ(Xdclin,XNMOBJ) THEN

--- O novo objeto OK, J existindo na(s) SADA(s), ou seja, no(s) XP_JA_OUT,
--- significa que NO DEVE NOVAMENTE SER ADICIONADO evitando duplicidade.

IF NOT Z_OK_XNMOBJ(Xrcsqlcmd,XNMOBJ) THEN


XP_JA_OUT := Z_JA_XNMOBJ(Xrcsqlcmd,XNMOBJ);
XPsenha := 'Update ' || XPNMTAB || ' ' || XPfetchs;

UPDATE Z_SQL_SQL_XREF
SET rcsqlcmd = XP_JA_OUT
WHERE ROWID = Xrowid;

END IF;

--- Guarda Rowid para teste de Recuperao Fetch aps UPDATE
XPTAGROWID := Xrowid;


END IF;

FETCH sqlCOMMANDS INTO
XNMOBJ
,Xnmarqjav
,Xnrlin
,Xdclin
,Xrctabvieseq
,Xrcprofun
,Xrcsqlcmd
,Xrcsyn
,Xrcclajpa
,Xrcclaact
,Xrowid;

END LOOP;
XPsenha := 'COMMIT ' || XPNMTAB;
COMMIT;
XPsenha := 'Close ' || XPNMTAB;
CLOSE sqlCOMMANDS;
----- END Cursor sqlCOMMANDS


return;

     EXCEPTION
       WHEN OTHERS THEN
         XPSENHA := XPSENHA || sqlerrm;
         return;

end;
/
